;;         x
;;      0     1    xsize=2
;;     ---- ----   ysize=3
;;  0 |    |    | 
;;    |    |    |     n
;; y   ---- ----      |
;;    |    |    |  w-- --e   
;;  1 |    |    |     |
;;     ---- ----      s
;;    |    |    |  
;;  2 |    |    |  
;;     ---- ----

(defclass 2dmap
  :super propertied-object
  :slots (the-map car-pos xsize ysize))
(defmethod 2dmap
  (:init 
   (x y &key ((:pos p) '(0 0)))
   (setq car-pos p xsize x ysize y)
   (setq the-map (make-map x y))
    self)
  ;;$B%^%C%W$H0LCV$rI=<($9$k(B
  (:show 
   (&key (show-pos t))
   (if show-pos
       (show-map the-map car-pos)
     (show-map the-map))
   )
  (:save-map
   (&optional (file-name "savemap.l"))
   (setq *export-map* (copy-object the-map))
   (dump-loadable-structure file-name *export-map*)
   )
  (:load-map
   (&optional (file-name "savemap.l"))
   (load file-name)
   (setq the-map (copy-object *export-map*))
   )
  (:set-pos
   (pos)
   (setq car-pos pos)
   )
  ;;set-pos$B$GM?$($?0LCV$N(Bdir$B$GM?$($?J}3Q$NJI$N>uBV$r%;%C%H$9$k(B
  ;;$B>uBV(Bwall$B$O(B -1:unknown 0:not exist 1:exist
  ;;(send *map* :set-wall 1 '(0 1) :west)
  (:set-wall
   (wall pos dire &key (set-pos t))
   (if set-pos
       (setq car-pos pos))
   (set-map the-map wall pos dire)
   )
  ;;get-pos$B$GM?$($?0LCV$N(Bdir$B$GM?$($?J}3Q$NJI$N$"$k$J$7$rF@$k(B
  ;;send *map* :get-wall '(0 1) :north
  (:get-wall
   (get-pos dire)
   (get-map the-map get-pos dire)
   )
  )

(defun make-map (xnum ynum)
  (let ((tmp (list)) (ret (list)))
    (dotimes (i ynum)
      (push (make-list (1+ xnum) :initial-element -1) tmp)
      )
    (push tmp ret)
    (setq tmp (list))
    (dotimes (i (1+ ynum))
      (push (make-list xnum :initial-element -1) tmp)
      )
    (push tmp ret)
    ret))
;; (make-list 2 :initial-element (make-list 2 :initial-element 0))
;; not equal  (list '(0 0) '(0 0))

(defun show-map-row (row)
  (let (mv)
    (format t ";;  ")
    (dotimes (x (length row))
      (format t " ")
      (setq mv (elt row x))
      (cond
       ((= mv -1)(format t "?????"))
       ((= mv 0) (format t "     "))
       ((= mv 1) (format t "-----"))
       (t (warn "unknown map value~A~%" mv)
	  (return-from show-map nil))
       )
      )
    (format t "~%")
    ))

(defun show-map-column (column &optional xpos)
  (let (str mv)
    (setq str ";;  ")
    (dotimes (x (length column))
      (setq mv (elt column x))
      (cond
       ((= mv -1)(setq str (concatenate string str "?")))
       ((= mv 0) (setq str (concatenate string str " ")))
       ((= mv 1) (setq str (concatenate string str "|")))
       (t (warn "unknown map value~A~%" mv)
	  (return-from show-map nil))
       )
      (cond
       ((eq xpos x)(setq str (concatenate string str " ### ")))
       (t (setq str (concatenate string str "     ")))
       )
      )
    (dotimes (i 2) (format t "~A~%" str))
    ))
    
    
(defun show-map (map &optional car-pos)
  (let ()
    (dotimes (y (length (second map)))
      (show-map-row (elt (car map) y))
      (if (and car-pos (= (elt car-pos 1) y))
	  (show-map-column (elt (second map) y) (elt car-pos 0))
	(show-map-column (elt (second map) y)))
      )
    (show-map-row (elt (car map) (1- (length (car map)))))
    nil)
  )

;; direction :north or :west or :south or :east
;; wall  -1 or  0 or 1
(defun set-map (map wall pos direction) 
  (let (rc xp yp)
    (case direction
      (:north (setq rc 0 xp (elt pos 0) yp (elt pos 1)))
      (:south (setq rc 0 xp (elt pos 0) yp (1+ (elt pos 1))))
      (:west  (setq rc 1 xp (elt pos 0) yp (elt pos 1)))
      (:east  (setq rc 1 xp (1+ (elt pos 0)) yp (elt pos 1)))
      (t (warn "unknown direction ~A~%" direction)
       (return-from set-map nil))
      )
    (setf (elt (elt (elt map rc) yp) xp) wall)
    )
  )

(defun get-map (map pos direction)
  (let (rc xp yp)
    (case direction
      (:north (setq rc 0 xp (elt pos 0) yp (elt pos 1)))
      (:south (setq rc 0 xp (elt pos 0) yp (1+ (elt pos 1))))
      (:west  (setq rc 1 xp (elt pos 0) yp (elt pos 1)))
      (:east  (setq rc 1 xp (1+ (elt pos 0)) yp (elt pos 1)))
      (t (warn "unknown direction ~A~%" direction)
	 (return-from set-map nil))
      )
    (elt (elt (elt map rc) yp) xp)
    )
  )